/* winthread.h
 * encapsulates the Windows thread
 * Copyright (C) 2003 Michel Leunen
 * http://www.leunen.com/
 * michel@leunen.com
 *
 * This software is freeware and is provided at no charge to the user.
 * This software is provided 'as-is', without any express or implied warranty.
 * To the maximum extent permitted by applicable law will the author NOT be held
 * liable for any damages whatsoever arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose, including
 * commercial applications, and to freely redistribute it, with the following
 * restrictions:
 *
 * 1. If you use this software in a product, an acknowledgment in the product
 *    documentation would be appreciated.
 *
 * 2. If you use this software in a commercial application, please send me an email
 *	  detailing the application.
 *
 * 3. This notice may not be removed or altered from any source distribution.
 *
 * 4. It may be distributed by any means, provided that the original files
 *    as supplied by the author remain intact and no charge is made other than
 *    for reasonable distribution costs. If you are a vendor, if you want to
 *    include this software in a compilation or in a magazine cover disk
 *    you HAVE TO CONTACT ME to obtain my authorization before distribution.
 */

#ifndef WINTHREAD_H
#define WINTHREAD_H
//---------------------------------------------------------------------------
#include <windows.h>

namespace mlLib
{

class CriticalSection
{
  private:
    CRITICAL_SECTION cs_;
  public:
    CriticalSection() {InitializeCriticalSection(&cs_);}
    ~CriticalSection() {DeleteCriticalSection(&cs_);}
    void acquire() {EnterCriticalSection(&cs_);}
    void release() {LeaveCriticalSection(&cs_);}
};

/**
 * Provide a safe way to use CriticalSection.
 * Imagine protecting data like this:
 *  CriticalSection protect;
 *  protect.acquire();
 *     //data to be protected
 *  protect.release();
 * If 'data to be protected' throws an exception, the critical
 * section will never be released. The class Lock avoid this
 * problem releasing the critical section when it goes out
 * of scope.
 */
class Lock
{
  private:
    CriticalSection &section_;
  public:
    Lock(CriticalSection &section) : section_(section) {section_.acquire();}
    ~Lock() {section_.release();}
};

/**
 * To use WinThread, you have to derive your own thread class
 * from WinThread and implement the Execute() method which is
 * where your thread code is executed.
 * Winthread is created in suspended mode. To run the thread,
 * call Resume().
 * The Execute() method should periodically check the state of
 * te Terminated property to know if another thread has requested
 * to terminate your thread.
 */
class WinThread
{
  private:
    HANDLE handle_;
    DWORD id_;
    bool suspended_;
    CriticalSection cs_;
    static DWORD __stdcall ThreadFunc(void* arg);
  protected:
    virtual void Execute()=0;
    bool Terminated;
  public:
    WinThread();
    virtual ~WinThread();
    bool Create();
    DWORD GetThreadId()const{return id_;}
    void Resume();
    void Suspend();
    void Terminate();
    void WaitFor(DWORD delay=INFINITE);
};
//---------------------------------------------------------------------------
} //end namespace mlLib
#endif
